#include "defines.h"

/*
 * 	The DCF77 time code
 *
 *
 *	SECOND		MEANING
 *
 *
 * 	 0 - 14		unused (always zero)
 *	15		alternate transmitter
 *	16		expect zone change (1 hour before)
 *	17              MEZ  (MET) 	    winter time
 *	18              MESZ (MED, MET DST) summer time
 *	19		leap insertion/deletion (1 hour before)
 *	20		start of time code (always 1)
 *	21 - 24		 1 Minutes
 *	25 - 27		10 Minutes
 *	28		Minute Parity
 *	29 - 32		 1 Hours
 *	33 - 34        	10 Hours
 *	35		Hour Parity
 *	36 - 39		 1 Days
 *	40 - 41		10 Days
 *	42 - 44		Day of week  Sunday .... Saturday
 *	45 - 48 	 1 Month
 *	49              10 Months
 *	50 - 53		 1 Years
 *	54 - 57		10 Years
 *	58		Date Parity
 *	59		Minute ready, except for leap insertion
 *
 *	all data are binary coded decimal, LSB first
 */



#define	Summer 		17	/* MET DST   	*/
#define Winter 		18	/* MET       	*/

#define DataStart	20

#define Min1Start	21
#define Min1End		24
#define Min10Start	25
#define Min10End	27
#define MinParity	28

#define Hour1Start	29 
#define Hour1End	32
#define Hour10Start	33
#define Hour10End	34
#define HourParity	35

#define Day1Start	36
#define Day1End		39
#define Day10Start	40
#define Day10End	41

#define WeekDayStart	42
#define WeekDayEnd	44


#define Month1Start	45
#define Month1End	48
#define Month10Start	49

#define Year1Start	50
#define Year1End	53
#define Year10Start	54
#define Year10End	57
#define DateParity	58

char *Month[] = {
	" ",
	"Jan",
	"Feb",
	"Mar",
	"Apr",	
	"May",
	"Jun",
	"Jul",
	"Aug",
	"Sep",
	"Oct",
	"Nov",
	"Dec",
	""
};

char *Day[] = {
	"Sun",
	"Mon",
	"Tue",
	"Wed",
	"Thu",
	"Fri",
	"Sat",
	""
};

int getnib (char *bp, int len)
{
	int	i, ret = 0, val = 1;

	for (i=0 ; i < len ; i++) {
		if (*bp) {
	 		ret += val;
		}
		val <<= 1;
		bp++;
	}
	return ret;
}

struct tm ParseDCF(char *buf)
{
	struct tm ts;
	
	ts.tm_sec  = 0;
	ts.tm_min  = getnib(buf+Min1Start,  4)+ 10 * getnib(buf+Min10Start,  3);
	ts.tm_hour = getnib(buf+Hour1Start, 4)+ 10 * getnib(buf+Hour10Start, 2);
	ts.tm_mday = getnib(buf+Day1Start,  4)+ 10 * getnib(buf+Day10Start,  2);
	ts.tm_mon  = getnib(buf+Month1Start,4)+ 10 * getnib(buf+Month10Start,2);
	ts.tm_year = getnib(buf+Year1Start, 4)+ 10 * getnib(buf+Year10Start, 4);
	ts.tm_wday = getnib(buf+WeekDayStart,3);
	ts.tm_yday = 0;
	ts.tm_isdst= 0;
	
	
	return (ts);
}


char *time2str(struct tm *buf, int set)
{
	static char	datestr[20];
	char		sstring[20];
	
	sprintf(datestr,"%s %02d. %s %02d, %02d:%02d:00",
			Day[buf->tm_wday],
			buf->tm_mday,
			Month[buf->tm_mon],
			buf->tm_year,
			buf->tm_hour,
			buf->tm_min);

	if (set) {
		sprintf(sstring,"/bin/date -s %02d:%02d",buf->tm_hour,buf->tm_min); 
		system(sstring);
	}

	return (datestr);
}


int CheckDCF(char *buf)
{
	int parity_flg; 

	parity_flg = TRUE;

	/*
	 *  Parity check and other error check possibility's
	 */

	if(!CheckParity(buf, Min1Start, MinParity))	/* Minutes Parity */
		parity_flg = FALSE;

	if(!CheckParity(buf, Hour1Start, HourParity))	/* Hour Parity */
		parity_flg = FALSE;

	if(!CheckParity(buf, Day1Start, DateParity))	/* Date Parity */
		parity_flg = FALSE;

	return (parity_flg);
}

/*--------------------------------------------------
 * Check the given range of 'buf' for EVEN parity.
 * Return TRUE if parity is ok.
 *--------------------------------------------------*/
 
int CheckParity(char *buf, int FirstByte, int ParityByte)
{
	char 	*tstr;
	int 	i, sum=0; 

	tstr   = buf + FirstByte;

	for( i = 0 ; i <= (ParityByte - FirstByte) ; i++ ) {
		if(*tstr == '1')
			sum ^= *tstr;
		tstr++; 
    	}
	if(sum)
		return(FALSE);
	else 	return(TRUE);
}
